use crate::config::config::STATIC_WS_PWD;
use crate::events::{Event, register_client, ClientMessage};
use crate::handles::close_connection::handle_connection_error;
use crate::handles::handle_messages::handle_other_message;
use crate::handles::handshake::handle_handshake;
use crate::handles::heartbeat::{handle_heartbeat, heart_resp};
use crate::handles::online_users_update::send_online_users_resp;
use crate::handles::redis::{insert_this_connection, remove_this_connection};
use crate::utils::json_utils::parse_message;
use futures::{SinkExt, StreamExt};
use lazy_static::lazy_static;
use std::collections::HashMap;
use std::sync::Arc;
use std::time::{Duration, Instant};
use tokio::sync::mpsc::{UnboundedSender, Sender, Receiver};
use tokio::sync::{mpsc, RwLock as AsyncRwLock};
use tokio::time;
use tokio_tungstenite::accept_hdr_async;
use tungstenite::handshake::server::{Request, Response};
use tungstenite::{Error, Message};

lazy_static! {
    pub static ref ONLINE_USERS: Arc<AsyncRwLock<HashMap<String, String>>> =
        Arc::new(AsyncRwLock::new(HashMap::new()));
}

// 关闭之前绑定的 WebSocket 连接
async fn close_existing_connection(event_sender: &UnboundedSender<Event>, from_id: &str) {
    event_sender.send(Event::CloseConnection(from_id.to_string())).unwrap();
    if let Err(e) = remove_this_connection(from_id).await {
        println!("从 Redis 中移除用户信息时出错: {}", e);
    }
}

pub(crate) async fn handle_client(
    stream: tokio::net::TcpStream,
    event_sender: UnboundedSender<Event>,
    center_to_client_sender: Sender<ClientMessage>,
    mut center_to_client_receiver: Receiver<ClientMessage>,
) -> Result<(), Error> {
    let must_existed_params = ["deviceId", "fromId", "wsPwd"];
    let mut connection_params = None;

    let ws_stream = match accept_hdr_async(stream, |req: &Request, resp| {
        match handle_handshake(req, &must_existed_params, STATIC_WS_PWD) {
            Ok(params) => {
                connection_params = Some(params);
                Ok(resp)
            }
            Err(error_msg) => {
                println!("{}", error_msg);
                let error_resp = Response::builder()
                    .status(400)
                    .header("Content-Type", "text/plain")
                    .body(Some(error_msg))
                    .unwrap();
                Err(error_resp)
            }
        }
    })
        .await
    {
        Ok(ws) => ws,
        Err(e) => {
            println!("WebSocket握手失败: {}", e);
            return Ok(());
        }
    };

    let (mut sender, mut receiver) = ws_stream.split();

    if let Some(params) = connection_params {
        if let Some(from_id) = params.get("fromId") {
            let from_id = from_id.clone();
            let from_id_clone = from_id.clone();

            // 检查 Redis 中是否已经存在该 fromId
            close_existing_connection(&event_sender, &from_id).await;

            // 将该用户的信息插入到 Redis 中
            if let Err(e) = insert_this_connection(&from_id, &params).await {
                println!("将用户信息插入到 Redis 中时出错: {}", e);
            }

            // 注册客户端到事件中心
            register_client(from_id.clone(), center_to_client_sender).await;

            // 发送新连接事件
            event_sender.send(Event::NewConnection(from_id.clone())).unwrap();

            let mut last_heartbeat_time = Instant::now();

            loop {
                tokio::select! {
                    // 处理消息接收
                    maybe_msg = receiver.next() => {
                        match maybe_msg {
                            Some(Ok(msg)) => {
                                if msg.is_text() {
                                    let text = msg.to_text().unwrap();
                                    match parse_message(text) {
                                        Ok(data) => {
                                            match data.msg_type.as_str() {
                                                "Heart" => {
                                                    println!("收到客户端心跳消息 {:?}", &data);
                                                    handle_heartbeat(&mut last_heartbeat_time);
                                                    if let Ok(json_str) = heart_resp(&from_id_clone) {
                                                        if let Err(e) = sender.send(Message::text(json_str)).await {
                                                            println!("发送心跳信息失败: {}", e);
                                                            // 发送关闭连接事件
                                                            event_sender.send(Event::CloseConnection(from_id_clone.clone())).unwrap();
                                                            handle_connection_error(&from_id_clone, &event_sender).await;
                                                            break;
                                                        }
                                                    }
                                                },
                                                "GetOnlineUserList" => {
                                                    println!("收到客户端获取在线用户列表 {:?}", &data);
                                                    if let Err(e) = send_online_users_and_send(&mut sender, &from_id_clone).await {
                                                        println!("处理在线用户列表出错了：{:?}", e);
                                                        // 发送关闭连接事件
                                                        event_sender.send(Event::CloseConnection(from_id_clone.clone())).unwrap();
                                                        handle_connection_error(&from_id_clone, &event_sender).await;
                                                        break;
                                                    }
                                                },
                                                _ => {
                                                    if let Err(e) = handle_other_message(&mut sender, &data, &from_id_clone).await {
                                                        println!("Failed to handle other message: {}", e);
                                                        // 发送关闭连接事件
                                                        event_sender.send(Event::CloseConnection(from_id_clone.clone())).unwrap();
                                                        handle_connection_error(&from_id_clone, &event_sender).await;
                                                        break;
                                                    }
                                                }
                                            }
                                        }
                                        Err(e) => {
                                            println!("解析JSON数据出错: {}", e);
                                        }
                                    }
                                }
                            }
                            Some(Err(e)) => {
                                println!("接受客户端消息出错: {}", e);
                                // 发送关闭连接事件
                                event_sender.send(Event::CloseConnection(from_id_clone.clone())).unwrap();
                                handle_connection_error(&from_id_clone, &event_sender).await;
                                break;
                            }
                            None => {
                                println!("客户端断开连接");
                                // 发送关闭连接事件
                                event_sender.send(Event::CloseConnection(from_id_clone.clone())).unwrap();
                                handle_connection_error(&from_id_clone, &event_sender).await;
                                break;
                            }
                        }
                    }
                    // 处理心跳超时
                    _ = time::sleep_until(tokio::time::Instant::from(last_heartbeat_time + Duration::from_secs(20))) => {
                        println!("用户id-{} 20秒内没有发送心跳，挂断连接", from_id_clone);
                        // 发送关闭连接事件
                        event_sender.send(Event::CloseConnection(from_id_clone.clone())).unwrap();
                        handle_connection_error(&from_id_clone, &event_sender).await;
                        break;
                    }
                    // 处理来自事件中心的消息
                    maybe_msg = center_to_client_receiver.recv() => {
                        if let Some(msg) = maybe_msg {
                            match msg {
                                ClientMessage::CmdUpdateOnlineUsers => {
                                    if let Err(e) = send_online_users_and_send(&mut sender, &from_id_clone).await {
                                        println!("处理在线用户列表出错了：{:?}", e);
                                        // 发送关闭连接事件
                                        event_sender.send(Event::CloseConnection(from_id_clone.clone())).unwrap();
                                        handle_connection_error(&from_id_clone, &event_sender).await;
                                        break;
                                    }
                                }
                                _ => {}
                            }
                        }
                    }
                }
            }
            println!("断开与用户id: {}，连接", from_id_clone);
            // 发送关闭连接事件
            event_sender.send(Event::CloseConnection(from_id_clone.clone())).unwrap();
            // 从 Redis 中移除该用户的信息
            if let Err(e) = remove_this_connection(&from_id_clone).await {
                println!("从 Redis 中移除用户信息时出错: {}", e);
            }
        }
    } else {
        println!("无法获取连接参数");
    }

    Ok(())
}

async fn send_online_users_and_send(sender: &mut (impl SinkExt<Message, Error = Error> + std::marker::Unpin), from_id: &str) -> Result<(), Error> {
    let messages = send_online_users_resp().await.map_err(|e| Error::Io(std::io::Error::new(std::io::ErrorKind::Other, e)))?;
    for (_, json) in messages {
        if let Err(e) = sender.send(Message::text(json)).await {
            println!("发送在线用户列表消息给用户 {} 失败: {}", from_id, e);
            return Err(e);
        }
    }
    Ok(())
}