use futures::SinkExt;
use lazy_static::lazy_static;
use std::collections::HashMap;
use std::sync::Arc;
use tokio::sync::mpsc;
use tokio::sync::RwLock;
use tungstenite::Error;

// 假设的用户详细信息结构体
#[derive(Debug)]
pub struct UserDetails {
    pub username: String,
}

// 定义事件类型
#[derive(Debug)]
pub enum Event {
    NewConnection(String),
    CloseConnection(String),
    UpdateOnlineUsers,
}

// 定义事件中心发送给客户端的消息类型
#[derive(Debug)]
pub enum ClientMessage {
    CmdUpdateOnlineUsers,
    SendMessage(String),
}

lazy_static! {
    // 存储每个客户端的发送者，用于事件中心向客户端发送消息
    pub static ref CLIENT_SENDERS: Arc<RwLock<HashMap<String, mpsc::Sender<ClientMessage>>>> =
        Arc::new(RwLock::new(HashMap::new()));
}

// 注册客户端的发送者
pub async fn register_client(from_id: String, center_to_client_sender: mpsc::Sender<ClientMessage>) {
    let mut senders = CLIENT_SENDERS.write().await;
    senders.insert(from_id, center_to_client_sender);
}

// 处理事件的任务
pub async fn handle_events(mut receiver: mpsc::UnboundedReceiver<Event>) {
    while let Some(event) = receiver.recv().await {
        match event {
            Event::NewConnection(from_id) => {
                println!("新连接: {}", from_id);
            }
            Event::CloseConnection(from_id) => {
                println!("关闭连接: {}", from_id);
                // 移除客户端的发送者
                let mut senders = CLIENT_SENDERS.write().await;
                senders.remove(&from_id);
            }
            Event::UpdateOnlineUsers => {
                // 这里可以实现其他触发更新在线用户列表的逻辑
                // 为简单起见，暂未详细实现
                println!("更新在线用户列表事件触发");
                notify_all_clients_to_update_online_users().await;
            }
        }
    }
}

// 向指定客户端发送消息
async fn send_message_to_client(from_id: &str, message: String) -> Result<(), Error> {
    let senders = CLIENT_SENDERS.read().await;
    if let Some(sender) = senders.get(from_id) {
        if let Err(e) = sender.send(ClientMessage::SendMessage(message)).await {
            return Err(Error::Io(std::io::Error::new(std::io::ErrorKind::Other, e)));
        }
    }
    Ok(())
}

// 通知所有客户端线程发送 CmdUpdateOnlineUsers 消息
async fn notify_all_clients_to_update_online_users() {
    let senders = CLIENT_SENDERS.read().await;
    for (_, sender) in senders.iter() {
        if let Err(e) = sender.send(ClientMessage::CmdUpdateOnlineUsers).await {
            println!("通知客户端更新在线用户列表失败: {:?}", e);
        }
    }
}